/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

#include "AccountDataServiceImpl_AccountDataService_Wrapper.h"

#include "osoa/sca/sca.h"

using services::accountdata::AccountDataServiceImpl;

extern "C"
{

    #if defined(WIN32) || defined(_WINDOWS)
    __declspec(dllexport) 
    #endif
    AccountDataServiceImpl_AccountDataService_Wrapper* AccountDataServiceImpl_AccountDataService_Wrapper_Factory(tuscany::sca::model::Service* target)
    {
        return new AccountDataServiceImpl_AccountDataService_Wrapper(target);
    }
}

AccountDataServiceImpl_AccountDataService_Wrapper::AccountDataServiceImpl_AccountDataService_Wrapper(tuscany::sca::model::Service* target) : tuscany::sca::cpp::CPPServiceWrapper(target)
{
    impl = (AccountDataServiceImpl*)getImplementation();
}

AccountDataServiceImpl_AccountDataService_Wrapper::~AccountDataServiceImpl_AccountDataService_Wrapper()
{
    releaseImplementation();
}

void* AccountDataServiceImpl_AccountDataService_Wrapper::newImplementation()
{
    return new AccountDataServiceImpl;
}

void AccountDataServiceImpl_AccountDataService_Wrapper::deleteImplementation()
{
    delete impl;
}

void AccountDataServiceImpl_AccountDataService_Wrapper::invokeService(tuscany::sca::Operation& operation)
{
    const std::string& operationName = operation.getName();

    if (operationName == "getCheckingAccount")
    {
        const std::string& p0 = *(const std::string*)operation.getParameterValue(0);

        if(operation.getReturnValue() != NULL)
        {
            *(commonj::sdo::DataObjectPtr*)operation.getReturnValue() = impl->getCheckingAccount(p0);
        }
        else
        {
            commonj::sdo::DataObjectPtr* ret = new commonj::sdo::DataObjectPtr;
            *ret = impl->getCheckingAccount(p0);
            operation.setReturnValue((const commonj::sdo::DataObjectPtr*)ret);
        }
        return;
    }
    if (operationName == "getSavingsAccount")
    {
        const std::string& p0 = *(const std::string*)operation.getParameterValue(0);

        if(operation.getReturnValue() != NULL)
        {
            *(commonj::sdo::DataObjectPtr*)operation.getReturnValue() = impl->getSavingsAccount(p0);
        }
        else
        {
            commonj::sdo::DataObjectPtr* ret = new commonj::sdo::DataObjectPtr;
            *ret = impl->getSavingsAccount(p0);
            operation.setReturnValue((const commonj::sdo::DataObjectPtr*)ret);
        }
        return;
    }
    if (operationName == "getStockAccount")
    {
        const std::string& p0 = *(const std::string*)operation.getParameterValue(0);

        if(operation.getReturnValue() != NULL)
        {
            *(commonj::sdo::DataObjectPtr*)operation.getReturnValue() = impl->getStockAccount(p0);
        }
        else
        {
            commonj::sdo::DataObjectPtr* ret = new commonj::sdo::DataObjectPtr;
            *ret = impl->getStockAccount(p0);
            operation.setReturnValue((const commonj::sdo::DataObjectPtr*)ret);
        }
        return;
    }
        

    throw osoa::sca::ServiceRuntimeException("Invalid operation");
    
}

